#!/usr/bin/env ruby
#:title: RubyLogo documentation
# = RubyLogo
# === A Logo interpreter written in Ruby
# Christian Kandler
# mailto:ckand002@student.informatik.fh-wiesbaden.de
#

# This is the RubyLogo startup script. It checks for available FXRuby modules
# and initializes a compatible GUI if possible. If no supported version of the
# Fox GUI Toolkit is found, it launches a primitive text mode interface instead.
#
begin
  require 'fox14'
  require 'LogoGUI'
  # everything is fine. Continue with the main GUI
  application = FXApp.new
  window = LogoGUI.new(application)
  application.create
  window.show(PLACEMENT_SCREEN)
  application.run

rescue LoadError
  # canid free zone. Use the cheap little text mode interface
  begin
    require 'LogoTurtle'
    require 'LogoParser'
  rescue LoadError => file1
    puts "#{file1}\nOne of the required files is missing. Make sure that the \
classes \'LogoTurtle\' and \'LogoParser\' are available"
    exit 1
  end

  print <<TITLE
+------------------------------------------------------------------------------+
|                  Welcome to the RubyLogo textmode interface                  |
+------------------------------------------------------------------------------+

Unfortunately neither fxruby1.4 nor the older fxruby1.0 module could be located
and the graphical user interface failed to start. Although you can't see it, the
RubyTurtle is here, and it's eagerly awaiting directions on where to move and
what to draw. Try entering a few commands at the prompt or install fxruby1.4
Enter an empty line to quit
TITLE
  tortoise = LogoTurtle.new
  lineparser = LogoParser.new(tortoise)
  while x = gets
    break if x =~ /^\s*$/
    lineparser.parse(x)
    puts tortoise.messages
  end
end


