#!/usr/bin/env ruby
# = RubyLogo
# === A Logo interpreter written in Ruby
# Christian Kandler
# mailto:ckand002@student.informatik.fh-wiesbaden.de
#

# A message window containing information and a manual for the RubyTurtle
# program
#
class LogoGUIInfoBox < Fox::FXDialogBox
  # Return a message window as a child window to the given parent.
  #
  def initialize(parent)
    super(parent, "Info", DECOR_ALL, 0, 0, 400, 400, 0, 0, 0, 0, 0, 0)
    self.icon = geticon(app)
    button = Fox::FXButton.new(self, "&Close help window", nil, self, ID_ACCEPT,
    LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X)
    textbox = Fox::FXText.new(self, nil, 0,
    LAYOUT_FILL_X|LAYOUT_FILL_Y|TEXT_READONLY)
    button.backColor = FXRGB(255, 200, 0)
    textbox.backColor = FXRGB(255, 255, 200)
    textbox.textColor = FXRGB(0, 0, 0)
    textbox.verticalScrollBar.backColor = FXRGB(255, 200, 0)
    textbox.horizontalScrollBar.backColor = FXRGB(255, 200, 0)
    textbox.text = <<INFO
RubyLogo - A Logo interpreter written in Ruby
(c) 2005, 2006 Christian Kandler


Refer to the included RDoc documentation for more information
about parser syntax and program details


The RubyLogo syntax:

forward x
move the turtle x steps forward

fd x
alias for forward x

backward x
move the turtle x steps backward

bk x
alias for backward x

right x
turn the turtle x degrees to the right

rt x
alias for right x

left x
turn the turtle x degrees to the left

lt x
alias for left x

move x y
move the turtle to the coordinates (x, y)

home
move the turtle to its starting coordinates

turn x
turn the turtle into the absolute direction x (clockwise
rotation, 0 being the starting direction)

reset
return turtle into its starting position, remove all drawn
lines but retain functions and variables

penup
raise pen from drawing area

pu
alias for penup

pendown
lower pen onto drawing area

pd
alias for pendown

hide
make the turtle invisible

show
make the turtle visible

color name
change the turtle's color to name

fg name
alias for color name

background name
change the background color to name

bg name
alias for background name

repeat x code end
repeat code for x times

if condition then code end
execute code only if condition is true

while condition do code end
execute code repeatedly as long as condition is true

def name [variables] code end
define a function with optional variables. Calling the function
through the command name will execute code. Functions must be
called with the correct number of arguments.

to name [variables] code end
alias for def name [variables] code end
INFO
  end

  # Returns a PNG image
  #
  def geticon(app)
helpimg = "\211PNG\r\n\032\n\000\000\000\rIHDR\000\000\000(\000\000\000(\010\
\000\000\000\000\251\225\347\261\000\000\003\032IDATx^}\324]lTU\030\206\321\247\
\273=\235\302\351\217S\nq\206\210vj bD\021!\332fL\323\"\020:\320fb\r\021\n\261\
\021\230\030IM \321\246!`lT.$*\332\230\210RM\224\004h\025\323i\255N\031#A0\344\
\213)&\030I\346\302\e\315@5\024\265t\nS\305\2753sv&\304u\363&_\336\354\213\223\
\234\027\261\215\036\332\331\034\252v\335\352PS\354\355Q\261\025\ty\227FN\3772K\
\216\272\273a\335}x$\347\263\315~\000\345\006\227,\t\272\n\240\374\251\343\222\
\223/\356\r\000\316\362X\337\300\350\330\330\350@_l\205\003\314\177\251\260\270\
\t\250h?\374\215x\016\267\225\003\321\204]L\264\000\215\375R\340H#\360\304\327^\
1\321\f\3051\271\215]\0164&\362\305\215\340\356\023m\354\235\035\221\306pK\347\
\253q\321z+\2415W\354\001\247W\264\327\227*\214`\247\371\216o\270\320m\212\0035\
\320\245\217\311m\n\317\303\203\372\370\"T\017\352b;4\213\026\303\306\n\363f\
\023DE\024\027\207\250\332\001\300\317\375\330\220/ul\257$>\016\362$l\021\355Y\
\264\332\350j\amiR\3377CTT\3724e-\000\\;\247\343\336\267z\016<\203\226\232\320\
\021)\343LZ\375\224\346\376\305\000L^\322\261z!\204\315\2233\277\352X\274\234\
\364Eu\001V\242]\017\314Q@\020\230\314\242\315\242=\n\027T\212\022S$\360\376\
\307\207^~nS-p\3264\234\273\320\036rHQKU\\\n\034\234\213\026J\210\026\257\342\
\036u\225\371>lp\264g\nm\203\037\315W\315dI\206\362\022l|r\020\243~=Fq\005\031\
\005\331\177\260\375\330\207\261\252\273\006\243H\201\3621\225\3056<\215\326\
\326\e\304 \373\027>U\305\3573X\256\216\353(\335\275w\0369\231+T\252\205\374}\
\031K\3467\035\035O\343\231\230\"\250B\3348\217%;\r\340\254\3032~\203\272\222ep\
\276\023O\331\276\233@i\r\226\357\341\001\206\0270\367\230\374\257cs\250\031B\
\332`\253xN\275\327w\313\aI\361l\203VQl(\345d\212\274?\367<\177\313\376\031\362\
R')\215\200\376\a\327H\336\347e\000\001k\241\232 \"\242\240c\036_\235 gV\001\
\024\221w\342\024\376\255\240 \024\2037\277\305@a\203\263\357\302\316:s\217F\
\270\376\312w\030\263\323\000\031\f\316\355\277\306\372vL\221\2560\177t\017\242\
9\r\253\376S\214\366E\367\004\217uy\3738R\017\264\016H\201\343\255\300#C\366\
\354%6\002\201\355G\305\362i\347\002`\355H\301\220\276P\t\270\341=\375\361D2\
\231\210\037\331\375\270\v\224\357\262\207\324\370h\255\017\300\361\327=\270,t\
\207\003\3404\177x\333\261\377a\370L\032\317\235\365kV\342\021[\374\300\226\206\
E\376\n\327\277(\334\361Z\\l\377\002\226\240&U\344=:h\000\000\000\000IEND\256B`\
\202"
    FXPNGIcon.new(app, helpimg, 0, 0, 40, 40)
  end
end


